package de.akquinet.jbosscc.needle;

import de.akquinet.jbosscc.needle.inject.Injector;
import de.akquinet.jbosscc.needle.seam.MockLog;
import de.akquinet.jbosscc.needle.seam.SeamInjector;
import org.jboss.seam.annotations.In;
import org.junit.Before;

import javax.persistence.EntityManager;
import java.lang.reflect.Field;
import java.util.List;

public abstract class AbstractTestcase<T> extends de.akquinet.jbosscc.needle.ejb.AbstractTestcase<T> {

    private MockLog mockLog;

    private SeamInjector injector = new SeamInjector();

    public AbstractTestcase() {
        super();
    }

    /**
     * @deprecated use no args c'tor
     */
    @Deprecated
    public AbstractTestcase(final Class<T> clazz) {
        super();
    }

    protected Injector newInjector() {
        return injector;
    }

    @Before
    public final void setupSeam() throws Exception {
        injectSeamFields();
    }

    private void injectSeamFields() {
        Object instance = getObjectUnderTest();

        mockLog = injector.injectSeamLogger(instance);

        List<Field> seamFields = injector.getAllSeamInjectedFields(instance);

        for (Field f : seamFields) {
            if (f.getType() == EntityManager.class) {
                injector.injectAllDeclaredFields(instance, In.class, lazyEmCreator,
                        new Injector.AnnotationChecker<In>() {
                            public boolean checkAnnotation(final Field field, final In annotation) {
                                return field.getType() == EntityManager.class;
                            }
                        });
            } else {
                Class type = f.getType();

                Object mock = createNiceMock(type);

                rememberMock(type, mock);

                injector.injectSeamIn(instance, f.getName(), mock);
            }
        }
    }
  
    protected void setLogVerbose(final boolean verbose) {
        if (mockLog != null) {
            mockLog.setVerbose(verbose);
        }
    }
}
